/***************************************************************************//**
 *   @file   AD5629R.c
 *   @brief  Implementation of AD5629R Driver.
 *   @author Dan Nechita
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 581
*******************************************************************************/

/******************************************************************************/
/* Include Files                                                              */
/******************************************************************************/
#include "AD5629R.h"		// AD5629R definitions.

/******************************************************************************/
/* Global Variables*/
/******************************************************************************/
unsigned char slaveAddrLSBs = 0;

/***************************************************************************//**
 * @brief Resets the device and performs several initializations.
 *
 * @param A1 - Used for setting bit 1 of the slave address. 
 * @param A0 - Used for setting bit 0 (LSB) of the slave address.
 *             Example: if A0 pin  - is connected to Vdd then A1 = 0; A0 = 0;
 *                                 - is not connected then    A1 = 1; A0 = 0;
 *                                 - is connected to GND then A1 = 1; A0 = 1;
 *
 * @return 1.
*******************************************************************************/

unsigned char AD5629R_Init(unsigned char A1, unsigned char A0)
{
    // Initializes communication.
    I2C_Init(); 
    // Sets the slave address corresponding to A0 pin connection.
    slaveAddrLSBs = (A1 << 1) | A0;
    AD5629R_LDAC_OUT;
    AD5629R_CLR_OUT;
    AD5629R_LDAC_LOW;   // Active low.
    AD5629R_CLR_HIGH;   // Active low.
    // Behaves as a power-on reset.
    AD5629R_Reset();
    // Clear code is set to 0x0000.
    AD5629R_SetInputRegister(AD5629R_CMD(AD5629R_CMD_LOAD_CLEAR_CODE)|
                             AD5629R_CODE_0X0000);

    return(1); 
}

/***************************************************************************//**
 * @brief Sets the device in a specific power mode.
 *
 * @param pwrMode - power mode of the device.
 *                  Example: AD5629R_PWRDN_NONE   - Normal operation
 *                           AD5629R_PWRDN_1K     - 1 kOhm to GND
 *                           AD5629R_PWRDN_100K   - 100 kOhm to GND
 *                           AD5629R_PWRDN_3STATE - Three-state
 *
 * @return none.
*******************************************************************************/
void AD5629R_PowerMode(unsigned char pwrMode)
{ 
    // Powers on all DAC channels.
    AD5629R_SetInputRegister(AD5629R_CMD(AD5629R_CMD_POWERDOWN) |
                             AD5629R_POWER_MODE(pwrMode) |
                             (1 << AD5629R_ADDR_DAC_A) |
                             (1 << AD5629R_ADDR_DAC_B) |
                             (1 << AD5629R_ADDR_DAC_C) |
                             (1 << AD5629R_ADDR_DAC_D) |
                             (1 << AD5629R_ADDR_DAC_E) |
                             (1 << AD5629R_ADDR_DAC_F) |
                             (1 << AD5629R_ADDR_DAC_G) |
                             (1 << AD5629R_ADDR_DAC_H));
}

/***************************************************************************//**
 * @brief Resets the device.
 *
 * @return none.
*******************************************************************************/
void AD5629R_Reset(void)
{
     AD5629R_SetInputRegister(AD5629R_CMD(AD5629R_CMD_RESET));
}

/***************************************************************************//**
 * @brief Writes a 24-bit data-word to the Input Register of the device.
 *
 * @param registerValue - Value of the register.
 *
 * @return none.
*******************************************************************************/
void AD5629R_SetInputRegister(unsigned long registerValue)
{
    unsigned char registerWord[3] = {0, 0, 0};
    unsigned char* dataPointer = (unsigned char*)&registerValue;

    registerWord[0] = dataPointer[2];
    registerWord[1] = dataPointer[1];
    registerWord[2] = dataPointer[0];
    I2C_Write(AD5629R_SLAVE_ADDR(slaveAddrLSBs), registerWord, 3, 1);
}

/***************************************************************************//**
 * @brief Reads the last register written by user.
 *
 * @return registerData - The content of the last written register.
*******************************************************************************/
unsigned short AD5629R_GetRegisterValue(void)
{
    unsigned char data[3] = {0, 0, 0};
    unsigned long registerData = 0x00;
    
    I2C_Read(AD5629R_SLAVE_ADDR(slaveAddrLSBs),data, 3, 1);
    registerData = ((unsigned long)data[0] << 16) + (data[1] << 8) + data[2];
    
    return(registerData);
}
